Attribute VB_Name = "modIShellItemImageFactory"
' Autor: F. Schler (frank@activevb.de)
' Datum: 09/2018

Option Explicit

Private Const S_OK As Long = &H0
Private Const IID_IPicture As String = "{7BF80980-BF32-101A-8BBB-00AA00300CAB}"
Private Const IID_IShellItemImageFactory As String = "{BCC18B79-BA16-442F-80C4-8A59C30C463B}"

Public Enum SIIGBF
    SIIGBF_RESIZETOFIT = &H0
    SIIGBF_BIGGERSIZEOK = &H1
    SIIGBF_MEMORYONLY = &H2
    SIIGBF_ICONONLY = &H4
    SIIGBF_THUMBNAILONLY = &H8
    SIIGBF_INCACHEONLY = &H10
    SIIGBF_CROPTOSQUARE = &H20
    SIIGBF_WIDETHUMBNAILS = &H40
    SIIGBF_ICONBACKGROUND = &H80
    SIIGBF_SCALEUP = &H100
End Enum

Public Type SIZE
    cx As Long
    cy As Long
End Type

Private Type IID
    Data1 As Long
    Data2 As Integer
    Data3 As Integer
    Data4(0 To 7) As Byte
End Type

Private Type PICTDESC
    cbSizeOfStruct As Long
    PicType As Long
    hgdiObj As Long
    hPalOrXYExt As Long
End Type

' ----==== Ole32 API-Deklarationen ====----
Private Declare Function IIDFromString Lib "ole32.dll" ( _
                         ByVal lpsz As Long, _
                         ByRef lpIID As IID) As Long

' ----==== Oleaut32 API-Deklarationen ====----
Private Declare Sub OleCreatePictureIndirect Lib "oleaut32.dll" ( _
                    ByRef lpPictDesc As PICTDESC, _
                    ByRef rIID As IID, _
                    ByVal fOwn As Boolean, _
                    ByRef lplpvObj As Object)

' ----==== Shell32 API-Deklarationen ====----
Private Declare Function SHCreateItemFromParsingName Lib "shell32.dll" ( _
                         ByVal pszPath As Long, _
                         ByVal pBC As Long, _
                         ByRef rIID As IID, _
                         ByRef pUnk As Long) As Long

Public Function GetThumbnail(ByVal FileFolder As String, ByVal ThumbSize As Long, _
    Optional ByVal Flags As SIIGBF = SIIGBF_RESIZETOFIT) As StdPicture

    Dim tIID As IID
    Dim tSIZE As SIZE
    Dim hBitmap As Long
    Dim pFileFolder As Long
    Dim oPic As StdPicture
    Dim pIShellItemImageFactory As Long
    Dim pIID_IShellItemImageFactory As Long
    Dim IShellItemImageFactory As clsIShellItemImageFactory

    If Len(FileFolder) <> 0 Then

        pIID_IShellItemImageFactory = StrPtr(IID_IShellItemImageFactory)

        If IIDFromString(pIID_IShellItemImageFactory, tIID) = S_OK Then

            pFileFolder = StrPtr(FileFolder)

            If SHCreateItemFromParsingName(pFileFolder, 0&, tIID, _
                pIShellItemImageFactory) = S_OK Then

                If pIShellItemImageFactory <> 0 Then

                    Set IShellItemImageFactory = New clsIShellItemImageFactory

                    Call IShellItemImageFactory.Initialize(pIShellItemImageFactory)

                    ThumbSize = Abs(ThumbSize)
                    
                    If ThumbSize > 1024 Then ThumbSize = 1024

                    With tSIZE

                        .cx = ThumbSize
                        .cy = ThumbSize

                    End With

                    hBitmap = IShellItemImageFactory.GetImage(tSIZE, Flags)

                    If hBitmap <> 0 Then
                    
                        Set oPic = HandleToPicture(hBitmap, vbPicTypeBitmap)

                    Else

                        Set oPic = Nothing

                    End If

                    Set IShellItemImageFactory = Nothing

                End If
            End If
        End If
    End If

    Set GetThumbnail = oPic

End Function

Private Function HandleToPicture(ByVal hGDIHandle As Long, ByVal ObjectType As _
    PictureTypeConstants, Optional ByVal hpal As Long = 0&) As StdPicture

    Dim tIID As IID
    Dim tPictDesc As PICTDESC
    Dim oPicture As IPicture
    Dim pIID_IPicture As Long

    pIID_IPicture = StrPtr(IID_IPicture)

    If IIDFromString(pIID_IPicture, tIID) = S_OK Then

        With tPictDesc

            .cbSizeOfStruct = Len(tPictDesc)
            .PicType = ObjectType
            .hgdiObj = hGDIHandle
            .hPalOrXYExt = hpal

        End With

        Call OleCreatePictureIndirect(tPictDesc, tIID, True, oPicture)

    End If

    Set HandleToPicture = oPicture

End Function

